// File: IOFactry.cc

#include <IOFactry.h>
#include <GeoObj.h>

IOFactory::IOFactory(MapOfPrototypes* _pPrototypes)
: pPrototypes(_pPrototypes), deleteStreams(FALSE),
  pIStream(NULL), pOStream(NULL)
{}

IOFactory::IOFactory(MapOfPrototypes* _pPrototypes,
                     istream* pis, ostream* pos)
: pPrototypes(_pPrototypes), deleteStreams(FALSE),
  pIStream(pis), pOStream(pos){}

IOFactory::IOFactory(MapOfPrototypes* _pPrototypes,
                     const String& ris, const String& ros)
: pPrototypes(_pPrototypes), deleteStreams(TRUE),
  pIStream(NULL), pOStream(NULL)
{
  if(ris != "")
  {
    pIStream = new ifstream(ris);
    if(!pIStream || !(*pIStream))
    {
      delete pIStream;
      pIStream = 0;
    }
  }
  if(ros != "")
  {
    pOStream = new ofstream(ros);
    if(!pOStream || !(*pOStream))
    {
      delete pOStream;
      pOStream = 0;
    }
  }
  deleteStreams = TRUE;
}

IOFactory::~IOFactory()
{
  if(deleteStreams)
  {
    delete pIStream;
    delete pOStream;
  }
}

GeoObj* IOFactory::CreateGeoObj(const String& TypeString)
{
  GeoObj* pProto = (*pPrototypes)[TypeString];
  GeoObj* pClone = NULL;

  if(pProto)
    pClone = pProto->Clone();
  else
  {
    cerr << "Error in GeoObj factory:"<< endl
         << "  could not find prototype for a "
         << TypeString << endl;
    return NULL;
  }

  if(pClone)
    _SaveGeoObj(pClone);
  else
  {
    cerr << "Error in GeoObj factory:"<< endl
         << "  could not clone prototype for a "
         << TypeString << endl;
    return NULL;
  }

  return pClone;
}
    

GeoObj* IOFactory::CreateGeoObj()
{
  if(pIStream && (*pIStream))
  {
    String TypeString;
    (*pIStream) >> TypeString;
   
    if(TypeString != "") 
      return CreateGeoObj(TypeString);
    else
      return NULL;
  }
  else
  {
    cerr << "Have no istream; returning NULL" << endl;
  }

  return NULL;
}

IOFactory::operator void*() const
{
  return (void*)(pIStream && (*pIStream));
}

void IOFactory::CollectGeoObj(GeoObj*& pg)
{
  if(pOStream && (*pOStream))
  {
    (*pOStream) << pg->GetTypeString();
    pg->WriteOn(*pOStream);
  }

  AbstractFactory::CollectGeoObj(pg);
}
